//
//  ViewController.swift
//  ToolSwift
//
//  Created by apple on 2024/1/17.
//

import UIKit
import SnapKit
import VisionCCiOSSDK

class ViewController: UIViewController {
    
    private let kFont:UIFont = .systemFont(ofSize: 16, weight: .regular)
    private let kColor:UIColor = .systemGray
    private let kMargin:CGFloat = 20
    private let kHeight:CGFloat = 44
    private let kTitleWidth:CGFloat = 80
    
    //MARK: - override
    override func viewDidLoad() {
        super.viewDidLoad()
        
        self.title = "维音Demo"
        self.view.backgroundColor = .white
        
        self.view.addSubview(self.labEntry)
        self.view.addSubview(self.txtEntry)
        
        self.view.addSubview(self.labHost)
        self.view.addSubview(self.txtHost)
        
        self.view.addSubview(self.labGuest)
        self.view.addSubview(self.txtGuest)
        
        self.view.addSubview(self.clickBtn)
        self.view.addSubview(self.clickBtnShow)
        self.view.addSubview(self.sendMessageBtn)
        self.view.addSubview(self.sendTextBtn)
        self.view.addSubview(self.sendImageBtn)
        
        self.view.addSubview(self.txtAppkey)
        self.view.addSubview(self.labVersion)
        
        if #unavailable(iOS 13.0){
            self.customUpdateConstarct()
        }
    }
    
    override func updateViewConstraints() {
        self.customUpdateConstarct()
        super.updateViewConstraints()
    }
    
    public override func viewWillAppear(_ animated: Bool) {
        super.viewWillAppear(animated)
        
        let _nav:UINavigationController? = self.navigationController
        _nav?.navigationBar.isTranslucent = false
        _nav?.isNavigationBarHidden = false
        _nav?.navigationBar.isHidden = false
        _nav?.toolbar.isHidden = false
        
        //启用IQKeyboardManager
        IQKeyboardManager.shared.enable = true
    }
    
    override func viewWillDisappear(_ animated: Bool) {
        super.viewWillDisappear(animated)
        
        //禁用IQKeyboardManager
        IQKeyboardManager.shared.enable = false
    }
    
    //MARK: lazy load
    /// 会话对象
    private lazy var conversionVC:CCKFApi = {
        return CCKFApi.init()
    }()
    
    /// 开始
    private lazy var clickBtn:UIButton = {[unowned self] in
        let _btn = UIButton(type: .custom)
        _btn.setTitle("开始(startSession)", for: .normal)
        _btn.setTitleColor(.white, for: .normal)
        _btn.backgroundColor = .systemBlue
        _btn.showsTouchWhenHighlighted = true
        _btn.layer.cornerRadius = 4
        _btn.addTarget(self, action: #selector(btnTestAction(sender:)), for: .touchUpInside)
        
        return _btn
    }()
    
    /// show
    private lazy var clickBtnShow:UIButton = {[unowned self] in
        let _btn = UIButton(type: .custom)
        _btn.setTitle("进入(show，非首次进入重复进入调用)", for: .normal)
        _btn.setTitleColor(.white, for: .normal)
        _btn.backgroundColor = .systemGreen
        _btn.showsTouchWhenHighlighted = true
        _btn.layer.cornerRadius = 4
        _btn.addTarget(self, action: #selector(btnTestAction(sender:)), for: .touchUpInside)
        
        return _btn
    }()
    
    /// 发送消息
    private lazy var sendMessageBtn:UIButton = {[unowned self] in
        let _btn = UIButton(type: .custom)
        _btn.setTitle("发送订单卡片", for: .normal)
        _btn.setTitleColor(.systemGray, for: .normal)
        _btn.showsTouchWhenHighlighted = true
        _btn.addTarget(self, action: #selector(btnSendMessageTest), for: .touchUpInside)
        
        return _btn
    }()
    
    /// 发送文本
    private lazy var sendTextBtn:UIButton = {[unowned self] in
        let _btn = UIButton(type: .custom)
        _btn.setTitle("发送文本", for: .normal)
        _btn.setTitleColor(.systemGray, for: .normal)
        _btn.showsTouchWhenHighlighted = true
        _btn.addTarget(self, action: #selector(btnSendTextTest), for: .touchUpInside)
        
        return _btn
    }()
    
    /// 发送图片
    private lazy var sendImageBtn:UIButton = {[unowned self] in
        let _btn = UIButton(type: .custom)
        _btn.setTitle("发送图片", for: .normal)
        _btn.setTitleColor(.systemGray, for: .normal)
        _btn.showsTouchWhenHighlighted = true
        _btn.addTarget(self, action: #selector(btnSendImageTest), for: .touchUpInside)
        
        return _btn
    }()
    
    /// 入口
    private lazy var labEntry:UILabel = {[unowned self] in
        let _lab = UILabel.init(frame: .zero)
        _lab.text = "入口ID"
        _lab.textColor = kColor
        _lab.font = kFont
        _lab.textAlignment = .left
        
        return _lab
    }()
    
    private lazy var txtEntry:UITextField = {[unowned self] in
        let _txt = UITextField.init(frame: .zero)
        _txt.font = kFont
        _txt.delegate = self
        _txt.returnKeyType = .send
        _txt.textColor = kColor
        _txt.borderStyle = .roundedRect
        _txt.text = "510540050086208"
        return _txt
    }()
    
    /// 域名
    private lazy var labHost:UILabel = {[unowned self] in
        let _lab = UILabel.init(frame: .zero)
        _lab.text = "域名Host"
        _lab.textColor = kColor
        _lab.font = kFont
        _lab.textAlignment = .left
        
        return _lab
    }()
    
    private lazy var txtHost:UITextField = {[unowned self] in
        let _txt = UITextField.init(frame: .zero)
        _txt.font = kFont
        _txt.delegate = self
        _txt.returnKeyType = .send
        _txt.textColor = kColor
        _txt.borderStyle = .roundedRect
        _txt.text = "https://vcc-sdk.vxish.cn"
        return _txt
    }()
    
    /// 用户配置
    private lazy var labGuest:UILabel = {[unowned self] in
        let _lab = UILabel.init(frame: .zero)
        _lab.text = "用户配置"
        _lab.textColor = kColor
        _lab.font = kFont
        _lab.textAlignment = .left
        
        return _lab
    }()
    
    private lazy var txtGuest:UITextView = {[unowned self] in
        let _txt = UITextView.init(frame: .zero)
        _txt.font = kFont
        _txt.delegate = self
        _txt.textColor = kColor
        _txt.returnKeyType = .send
        _txt.layer.cornerRadius = 5
        _txt.layer.borderWidth = 1
        _txt.layer.borderColor = UIColor.lightGray.cgColor
        _txt.text = "{\"identity_id\":\"2024\",\"visitor_name\":\"谁家小可爱\",\"phone\":\"110\",\"email\":\"110@qq.com\"}"
        return _txt
    }()
    
    /// appKey
    private lazy var txtAppkey:UITextView = {[unowned self] in
        let _txt = UITextView.init(frame: .zero)
        _txt.font = kFont
        _txt.delegate = self
        _txt.returnKeyType = .send
        _txt.toolbarPlaceholder = "请输入appkey"
        _txt.textColor = kColor
        _txt.layer.cornerRadius = 5
        _txt.layer.borderWidth = 1
        _txt.layer.borderColor = UIColor.lightGray.cgColor
        _txt.text = "jbGllbnRfc2UiLCJjbGllbnRfbWEiLCJjbGllbnRfaXZyIiwiY2xpZW50X3JvYm90IiwiY2xpZW50X2NoYXQiLCJjbGllbnRfYXN5bmN0YXNrIiwiY2xpZW50X21vbm"
        return _txt
    }()
    
    /// 版本
    private lazy var labVersion:UILabel = {[unowned self] in
        let _lab = UILabel.init(frame: .zero)
        _lab.text = "VisionCCiOSSDK-V:" + VisionCCiOSSDK.CCKFApi.getSDKVersion()
        _lab.textColor = kColor
        _lab.font = kFont
        _lab.textAlignment = .center
        
        return _lab
    }()
}


//MARK: -
extension ViewController {
    
    //MARK: 测试
    /// 测试
    @objc func btnTestAction(sender:UIButton?) {
        if sender == self.clickBtn {
            guard let _entry = self.txtEntry.text,!_entry.isEmpty else {
                SVProgressHUD.show(withStatus: "入口Id不存在")
                return
            }
            
            guard let _host = self.txtHost.text,!_host.isEmpty else {
                SVProgressHUD.show(withStatus: "域名Host不存在")
                return
            }
            
            guard let _appKey = self.txtAppkey.text,_appKey.isEmpty == false else {
                SVProgressHUD.show(withStatus: "appKey不存在")
                return
            }
            
            if let _data = self.txtGuest.text.data(using: .utf8) {
                do{
                    let _result = try JSONDecoder.init().decode(UserMappingModel.self, from: _data)
                    
                    weak var weakSelf = self
                    self.conversionVC.conversionDelegate = weakSelf
                    self.conversionVC.startSession(host: _host,
                                                   entryId: _entry,
                                                   appkey: _appKey,
                                                   userMappings: _result) {[weak self] in
                        guard let self = self else {return}
                        
                        //进线卡片
                        let _jsonString = """
                {"cardType":2,"customItems":[{"customCardName":"CLIMACOOL清风鞋」减震耐磨运动鞋男女adidas阿迪达斯轻运动","customCardDesc":"采用运动风设计 穿着舒适","customCardLink":"","customCardThumbnail":"https://static1.adidas.com.cn/t395/MTcwMTIyMDk1NTc2NzNhYjQyM2FjLWVlNjQtNGU0YS05NGM1.jpeg?im=Resize=(200,200)","customCardAmountName":"价格","customCardAmountSymbol":"￥","customCardAmount":"478","customCardOriginalAmount":"500"}],"customFields":[{"key":"颜色","value":"黑色"},{"key":"货号","value":"ID6926"}]}
                """
                        if let _data = _jsonString.data(using: .utf8) {
                            do {
                                let _msgBody = try JSONDecoder.init().decode(MessageBody.self, from: _data)
                                self.conversionVC.sendMessage(msgType: 10, msgBody: _msgBody)
                            }
                            catch(let _error){
                                SVProgressHUD.showError(withStatus: _error.localizedDescription)
                            }
                        }
                    }
                    self.navigationController?.pushViewController(self.conversionVC, animated: true)
                }
                catch(let _error){
                    debugPrint(_error)
                    SVProgressHUD.show(withStatus: _error.localizedDescription)
                }
            }
        }
        //show
        else{
            self.conversionVC.show()
            self.navigationController?.pushViewController(self.conversionVC, animated: true)
        }
    }
    
    
    //MARK: 发送消息
    /// 发送订单卡片消息
    @objc func btnSendMessageTest(){
        
        let _jsonString = """
{
    "cardType":1,
    "cardGuide":"订单：123456789",
    "cardDesc":"2024-01-13 23:59:59",
    "cardLink":"",
    "cardImg":"",
    "customItems":[
        {
            "customCardName":"CLIMACOOL清风鞋」减震耐磨运动鞋男女adidas阿迪达斯轻运动",
            "customCardDesc":"",
            "customCardLink":"",
            "customCardThumbnail":"https://static1.adidas.com.cn/t395/MTcwMTIyMDk1NTc2NzNhYjQyM2FjLWVlNjQtNGU0YS05NGM1.jpeg?im=Resize=(200,200)",
            "customCardAmountName":"4件商品 总计",
            "customCardAmountSymbol":"￥",
            "customCardAmount":"1230.00",
            "customCardOriginalAmount":""
        }
    ],
    "customFields":[
        {
            "key":"订单状态",
            "value":"待评价"
        }
    ]
}
"""
        if let _data = _jsonString.data(using: .utf8) {
            do {
                let _msgBody = try JSONDecoder.init().decode(MessageBody.self, from: _data)
                self.conversionVC.sendMessage(msgType: 10, msgBody: _msgBody)
                self.navigationController?.pushViewController(self.conversionVC, animated: true)
            }
            catch(let _error){
                SVProgressHUD.showError(withStatus: _error.localizedDescription)
            }
        }
        else{
            SVProgressHUD.showError(withStatus: "获取Data失败")
        }
    }
    
    /// 发送文本消息
    @objc func btnSendTextTest(){
        
        let _jsonString = """
{"content":"🙏🏻值此2024🐲年新春佳节，㊗️您阖家欢乐万事如意，心想事成身体安康"}
"""
        if let _data = _jsonString.data(using: .utf8) {
            do {
                let _msgBody = try JSONDecoder.init().decode(MessageBody.self, from: _data)
                self.conversionVC.sendMessage(msgType: 1, msgBody: _msgBody)
                self.navigationController?.pushViewController(self.conversionVC, animated: true)
            }
            catch(let _error){
                SVProgressHUD.showError(withStatus: _error.localizedDescription)
            }
        }
        else{
            SVProgressHUD.showError(withStatus: "获取Data失败")
        }
    }
    
    /// 发送图片消息
    @objc func btnSendImageTest(){
        
        let _jsonString = """
{"mediaUrl":"https://img0.baidu.com/it/u=1765134239,2641237103&fm=253&app=138&size=w931&n=0&f=JPEG&fmt=auto?sec=1708448400&t=b05cade91ccedb65279bae5bedf9115e"}
"""
        if let _data = _jsonString.data(using: .utf8) {
            do {
                let _msgBody = try JSONDecoder.init().decode(MessageBody.self, from: _data)
                self.conversionVC.sendMessage(msgType: 2, msgBody: _msgBody)
                self.navigationController?.pushViewController(self.conversionVC, animated: true)
            }
            catch(let _error){
                SVProgressHUD.showError(withStatus: _error.localizedDescription)
            }
        }
        else{
            SVProgressHUD.showError(withStatus: "获取Data失败")
        }
    }
    
    //MARK: 布局
    private func customUpdateConstarct(){
        if self.view.subviews.contains(self.labEntry){
            self.labEntry.snp.makeConstraints {[weak self] make in
                guard let self = self else { return }
                make.left.equalTo(kMargin)
                make.width.equalTo(kTitleWidth)
                make.height.equalTo(kHeight)
                make.top.equalTo(30)
            }
        }
        
        if self.view.subviews.contains(self.txtEntry){
            self.txtEntry.snp.makeConstraints {[weak self] make in
                guard let self = self else { return }
                make.left.equalTo(self.labEntry.snp.right).offset(10)
                make.right.equalTo(-kMargin)
                make.height.equalTo(kHeight)
                make.centerY.equalTo(self.labEntry.snp.centerY)
            }
        }
        
        //MARK: -
        if self.view.subviews.contains(self.labHost){
            self.labHost.snp.makeConstraints {[weak self] make in
                guard let self = self else { return }
                make.left.equalTo(kMargin)
                make.width.equalTo(kTitleWidth)
                make.height.equalTo(kHeight)
                make.top.equalTo(self.labEntry.snp.bottom).offset(10)
            }
        }
        
        if self.view.subviews.contains(self.txtHost){
            self.txtHost.snp.makeConstraints {[weak self] make in
                guard let self = self else { return }
                make.left.equalTo(self.txtEntry.snp.left)
                make.right.equalTo(-kMargin)
                make.height.equalTo(kHeight)
                make.centerY.equalTo(self.labHost.snp.centerY)
            }
        }
        
        //MARK: -
        if self.view.subviews.contains(self.labGuest){
            self.labGuest.snp.makeConstraints {[weak self] make in
                guard let self = self else { return }
                make.left.equalTo(kMargin)
                make.width.equalTo(kTitleWidth)
                make.height.equalTo(kHeight)
                make.top.equalTo(self.labHost.snp.bottom).offset(10)
            }
        }
        
        if self.view.subviews.contains(self.txtGuest){
            self.txtGuest.snp.makeConstraints {[weak self] make in
                guard let self = self else { return }
                make.left.equalTo(self.txtEntry.snp.left)
                make.right.equalTo(-kMargin)
                make.height.equalTo(2 * kHeight)
                make.top.equalTo(self.labGuest.snp.top)
            }
        }
        
        //MARK: -
        if self.view.subviews.contains(self.txtAppkey){
            self.txtAppkey.snp.makeConstraints { [weak self] make in
                guard let self = self else { return }
                make.left.equalTo(kMargin)
                make.right.equalTo(-kMargin)
                make.height.equalTo(2 * kHeight)
                make.top.equalTo(self.txtGuest.snp.bottom).offset(10)
            }
        }
        
        //MARK: -
        if self.view.subviews.contains(self.clickBtn){
            self.clickBtn.snp.makeConstraints {[weak self] make in
                guard let self = self else { return }
                make.left.equalTo(kMargin)
                make.right.equalTo(-kMargin)
                make.top.equalTo(self.txtAppkey.snp.bottom).offset(20)
                make.height.equalTo(kHeight)
            }
        }
        
        if self.view.subviews.contains(self.clickBtnShow){
            self.clickBtnShow.snp.makeConstraints {[weak self] make in
                guard let self = self else { return }
                make.left.equalTo(kMargin)
                make.right.equalTo(-kMargin)
                make.top.equalTo(self.clickBtn.snp.bottom).offset(10)
                make.height.equalTo(kHeight)
            }
        }
        
        if self.view.subviews.contains(self.sendMessageBtn){
            self.sendMessageBtn.snp.makeConstraints {[weak self] make in
                guard let self = self else { return }
                make.width.equalTo(120)
                make.height.equalTo(30)
                make.right.equalTo(self.clickBtn.snp.right)
                make.top.equalTo(self.clickBtnShow.snp.bottom).offset(10)
            }
        }
        
        if self.view.subviews.contains(self.sendTextBtn){
            self.sendTextBtn.snp.makeConstraints {[weak self] make in
                guard let self = self else { return }
                make.width.equalTo(80)
                make.height.equalTo(30)
                make.left.equalTo(self.clickBtn.snp.left)
                make.top.equalTo(self.clickBtnShow.snp.bottom).offset(10)
            }
        }
        
        if self.view.subviews.contains(self.sendImageBtn){
            self.sendImageBtn.snp.makeConstraints {[weak self] make in
                guard let self = self else { return }
                make.width.equalTo(80)
                make.height.equalTo(30)
                make.centerX.equalToSuperview()
                make.top.equalTo(self.clickBtnShow.snp.bottom).offset(10)
            }
        }
        
        if self.view.subviews.contains(self.labVersion){
            self.labVersion.snp.makeConstraints { make in
                make.left.right.equalTo(0)
                make.height.equalTo(21)
                make.bottom.equalTo(-60)
            }
        }
    }
    
}

//MARK: -
extension ViewController : UITextFieldDelegate {
    
    func textField(_ textField: UITextField, shouldChangeCharactersIn range: NSRange, replacementString string: String) -> Bool {
        //发送
        if string == "\n" {
            self.btnTestAction(sender: self.clickBtn)
            return false
        }
        
        return true
    }
}


//MARK: - UITextViewDelegate
extension ViewController : UITextViewDelegate {
    
    func textView(_ textView: UITextView, shouldChangeTextIn range: NSRange, replacementText text: String) -> Bool {
        //发送
        if text == "\n" {
            self.btnTestAction(sender: self.clickBtn)
            return false
        }
        
        return true
    }
    
}


//MARK: - 事件订阅
/// 事件订阅
extension ViewController : CCKFApiConversationDelegate {
    
    /// 主动推送未读消息的数量
    func unReadMessageCountEvent(count: Int) {
        debugPrint(count)
    }
    
    /// 快捷语传递给上层应用
    func shortCutEvent(model: VisionCCiOSSDK.ShortcutsModel) {
        debugPrint(model)
    }
    
    /// 有消息更新
    func messageEvent(model: VisionCCiOSSDK.MessageModel) {
        debugPrint(model)
    }
    
}
